@@ -803,16 +803,10 @@ list_candidates(
         }
 
         /*
-         * Assert we recieved a valid idl. If it was NULL, it means somewhere we failed
-         * during the dblayer interactions.
-         *
-         * idl_set requires a valid idl structure to generate the linked list of
-         * idls that we insert.
+         * The IDL for that component is NULL, so no candidate retrieved from that component. This is all normal
+         * Just build a idl with an empty set
          */
         if (tmp == NULL) {
-            slapi_log_err(SLAPI_LOG_CRIT, "list_candidates", "NULL idl was recieved from filter_candidates_ext.");
-            slapi_log_err(SLAPI_LOG_CRIT, "list_candidates", "Falling back to empty IDL set. This may affect your search results.");
-            PR_ASSERT(tmp);
             tmp = idl_alloc(0);
         }
 
@@ -974,7 +968,7 @@ keys2idl(
     IDList *idl = NULL;
 
     slapi_log_err(SLAPI_LOG_TRACE, "keys2idl", "=> type %s indextype %s\n", type, indextype);
-    for (size_t i = 0; ivals[i] != NULL; i++) {
+    for (uint32_t i = 0; ivals[i] != NULL; i++) {
         IDList *idl2 = NULL;
 
         idl2 = index_read_ext_allids(pb, be, type, indextype, slapi_value_get_berval(ivals[i]), txn, err, unindexed, allidslimit);
@@ -985,12 +979,12 @@ keys2idl(
             char buf[BUFSIZ];
 
             slapi_log_err(SLAPI_LOG_TRACE, "keys2idl",
-                          "   ival[%lu] = \"%s\" => %lu IDs\n", i,
-                          encode(slapi_value_get_berval(ivals[i]), buf), (u_long)IDL_NIDS(idl2));
+                          "   ival[%" PRIu32 "] = \"%s\" => %" PRIu32 " IDs\n", i,
+                          encode(slapi_value_get_berval(ivals[i]), buf), (uint32_t)IDL_NIDS(idl2));
         }
 #endif
         if (idl2 == NULL) {
-            slapi_log_err(SLAPI_LOG_WARNING, "keys2idl", "recieved NULL idl from index_read_ext_allids, treating as empty set\n");
+            slapi_log_err(SLAPI_LOG_WARNING, "keys2idl", "received NULL idl from index_read_ext_allids, treating as empty set\n");
             slapi_log_err(SLAPI_LOG_WARNING, "keys2idl", "this is probably a bug that should be reported\n");
             idl2 = idl_alloc(0);
         }
