@@ -83,10 +83,10 @@ idl_new_get_allidslimit(struct attrinfo *a, int allidslimit)
 }
 
 int
-idl_new_exceeds_allidslimit(size_t count, struct attrinfo *a, int allidslimit)
+idl_new_exceeds_allidslimit(uint64_t count, struct attrinfo *a, int allidslimit)
 {
-    size_t limit = idl_new_get_allidslimit(a, allidslimit);
-    return (limit != (size_t)-1) && (count > limit);
+    uint64_t limit = idl_new_get_allidslimit(a, allidslimit);
+    return (limit != (uint64_t)-1) && (count > limit);
 }
 
 
@@ -138,7 +138,7 @@ idl_new_fetch(
     DBT key;
     DBT data;
     ID id = 0;
-    size_t count = 0;
+    uint64_t count = 0;
     /* beware that a large buffer on the stack might cause a stack overflow on some platforms */
     char buffer[BULK_FETCH_BUFFER_SIZE];
     void *ptr;
@@ -245,7 +245,7 @@ idl_new_fetch(
             count++;
         }
 
-        slapi_log_err(SLAPI_LOG_TRACE, "idl_new_fetch", "bulk fetch buffer nids=%lu\n", count);
+        slapi_log_err(SLAPI_LOG_TRACE, "idl_new_fetch", "bulk fetch buffer nids=%" PRIu64 "\n", count);
 #if defined(DB_ALLIDS_ON_READ)
         /* enforce the allids read limit */
         if ((NEW_IDL_NO_ALLID != *flag_err) && (NULL != a) &&
@@ -254,7 +254,7 @@ idl_new_fetch(
             idl->b_ids[0] = ALLID;
             ret = DB_NOTFOUND; /* fool the code below into thinking that we finished the dups */
             slapi_log_err(SLAPI_LOG_BACKLDBM, "idl_new_fetch", "Search for key for attribute index %s "
-                                                               "exceeded allidslimit %d - count is %lu\n",
+                                                               "exceeded allidslimit %d - count is %" PRIu64 "\n",
                           a->ai_type, allidslimit, count);
             break;
         }
@@ -320,6 +320,9 @@ typedef struct _range_id_pair
  * In the total update (bulk import), an entry requires its ancestors already added.
  * To guarantee it, the range search with parentid is used with setting the flag
  * SLAPI_OP_RANGE_NO_IDL_SORT in operator.
+ * In bulk import the range search is parentid>=1 to retrieve all the entries
+ * But we need to order the IDL with the parents first => retrieve the suffix entry ID
+ * to store the children
  *
  * If the flag is set,
  * 1. the IDList is not sorted by the ID.
@@ -348,7 +351,7 @@ idl_new_range_fetch(
     DBT cur_key = {0};
     DBT data = {0};
     ID id = 0;
-    size_t count = 0;
+    uint64_t count = 0;
     /* beware that a large buffer on the stack might cause a stack overflow on some platforms */
     char buffer[BULK_FETCH_BUFFER_SIZE];
     void *ptr;
@@ -366,6 +369,23 @@ idl_new_range_fetch(
     if (NULL == flag_err) {
         return NULL;
     }
+    if (operator & SLAPI_OP_RANGE_NO_IDL_SORT) {
+            struct _back_info_index_key bck_info;
+            int rc;
+            /* We are doing a bulk import
+             * try to retrieve the suffix entry id from the index
+             */
+
+            bck_info.index = SLAPI_ATTR_PARENTID;
+            bck_info.key = "0";
+
+            if (rc = slapi_back_get_info(be, BACK_INFO_INDEX_KEY, (void **)&bck_info)) {
+                slapi_log_err(SLAPI_LOG_WARNING, "idl_new_range_fetch", "Total update: fail to retrieve suffix entryID, continue assuming it is the first entry\n");
+            }
+            if (bck_info.key_found) {
+                suffix = bck_info.id;
+            }
+    }
 
     if (NEW_IDL_NOOP == *flag_err) {
         return NULL;
@@ -455,7 +475,7 @@ idl_new_range_fetch(
             *flag_err = LDAP_TIMELIMIT_EXCEEDED;
             goto error;
         }
-        if (operator&SLAPI_OP_RANGE_NO_IDL_SORT) {
+        if (operator & SLAPI_OP_RANGE_NO_IDL_SORT) {
             key = (ID)strtol((char *)cur_key.data + 1, (char **)NULL, 10);
         }
         while (PR_TRUE) {
@@ -487,9 +507,13 @@ idl_new_range_fetch(
             /* note the last id read to check for dups */
             lastid = id;
             /* we got another ID, add it to our IDL */
-            if (operator&SLAPI_OP_RANGE_NO_IDL_SORT) {
-                if (count == 0) {
-                    /* First time.  Keep the suffix ID. */
+            if (operator & SLAPI_OP_RANGE_NO_IDL_SORT) {
+                if ((count == 0) && (suffix == 0)) {
+                    /* First time.  Keep the suffix ID. 
+                     * note that 'suffix==0' mean we did not retrieve the suffix entry id
+                     * from the parentid index (key '=0'), so let assume the first
+                     * found entry is the one from the suffix
+                     */
                     suffix = key;
                     idl_rc = idl_append_extend(&idl, id);
                 } else if ((key == suffix) || idl_id_is_in_idlist(idl, key)) {
@@ -522,7 +546,7 @@ idl_new_range_fetch(
         }
 
         slapi_log_err(SLAPI_LOG_TRACE, "idl_new_range_fetch",
-                      "Bulk fetch buffer nids=%lu\n", count);
+                      "Bulk fetch buffer nids=%" PRIu64 "\n", count);
 #if defined(DB_ALLIDS_ON_READ)
         /* enforce the allids read limit */
         if ((NEW_IDL_NO_ALLID != *flag_err) && ai && (idl != NULL) &&
@@ -615,9 +639,11 @@ error:
     }
     if (operator&SLAPI_OP_RANGE_NO_IDL_SORT) {
         size_t remaining = leftovercnt;
+
         while(remaining > 0) {
             for (size_t i = 0; i < leftovercnt; i++) {
                 if (leftover[i].key > 0 && idl_id_is_in_idlist(idl, leftover[i].key) != 0) {
+                    /* if the leftover key has its parent in the idl */
                     idl_rc = idl_append_extend(&idl, leftover[i].id);
                     if (idl_rc) {
                         slapi_log_err(SLAPI_LOG_ERR, "idl_new_range_fetch",
