@@ -376,7 +376,7 @@ char *slapi_dn_normalize_case_original(char *dn);
 int32_t ndn_cache_init(void);
 void ndn_cache_destroy(void);
 int ndn_cache_started(void);
-void ndn_cache_get_stats(PRUint64 *hits, PRUint64 *tries, size_t *size, size_t *max_size, size_t *thread_size, size_t *evicts, size_t *slots, long *count);
+void ndn_cache_get_stats(uint64_t *hits, uint64_t *tries, uint64_t *size, uint64_t *max_size, uint64_t *thread_size, uint64_t *evicts, uint64_t *slots, uint64_t *count);
 #define NDN_DEFAULT_SIZE 20971520 /* 20mb - size of normalized dn cache */
 
 /* filter.c */
@@ -1171,13 +1171,44 @@ int slapd_security_library_is_initialized(void);
 int slapd_nss_is_initialized(void);
 char *slapd_get_tmp_dir(void);
 
+/* thread-data.c */
+/* defines for internal logging */
+typedef enum _slapi_op_nest_state {
+    OP_STATE_NOTNESTED = 0,
+    OP_STATE_NESTED = 1,
+    OP_STATE_UNNESTED = 2,
+} slapi_log_nest_state;
+
+
+struct slapi_td_log_op_state_t {
+    int32_t op_id;
+    int32_t op_int_id;
+    int32_t op_nest_count;
+    slapi_log_nest_state op_nest_state;
+    int64_t conn_id;
+};
+
+int slapi_td_init(void);
+int slapi_td_set_dn(char *dn);
+void slapi_td_get_dn(char **dn);
+int slapi_td_plugin_lock_init(void);
+int slapi_td_get_plugin_locked(void);
+int slapi_td_set_plugin_locked(void);
+int slapi_td_set_plugin_unlocked(void);
+struct slapi_td_log_op_state_t * slapi_td_get_log_op_state(void);
+void slapi_td_internal_op_start(void);
+void slapi_td_internal_op_finish(void);
+void slapi_td_reset_internal_logging(uint64_t conn_id, int32_t op_id);
+
+/*  Thread Local Storage Index Types - thread_data.c */
+
 /* util.c */
 #include <stdio.h> /* GGOODREPL - For BUFSIZ, below, gak */
 const char *escape_string(const char *str, char buf[BUFSIZ]);
 const char *escape_string_with_punctuation(const char *str, char buf[BUFSIZ]);
 const char *escape_string_for_filename(const char *str, char buf[BUFSIZ]);
 void strcpy_unescape_value(char *d, const char *s);
-
+void get_internal_conn_op (uint64_t *connid, int32_t *op_id, int32_t *op_internal_id);
 char *slapi_berval_get_string_copy(const struct berval *bval);
 
 /* lenstr stuff */
