@@ -161,6 +161,11 @@ do_escape_string(
                     break;
                 }
                 do {
+                    if (bufSpace < 4) {
+                        memcpy(bufNext, "..", 2);
+                        bufNext += 2;
+                        goto bail;
+                    }
                     if (esc == UTIL_ESCAPE_BACKSLASH) {
                         /* *s is '\\' */
                         /* If *(s+1) and *(s+2) are both hex digits,
@@ -179,11 +184,6 @@ do_escape_string(
                             *bufNext++ = '\\';
                             --bufSpace;
                         }
-                        if (bufSpace < 3) {
-                            memcpy(bufNext, "..", 2);
-                            bufNext += 2;
-                            goto bail;
-                        }
                         PR_snprintf(bufNext, 3, "%02x", *(unsigned char *)s);
                         bufNext += 2;
                         bufSpace -= 2;
@@ -1178,7 +1178,7 @@ slapd_chown_if_not_owner(const char *filename, uid_t uid, gid_t gid)
 }
 
 /*
- * Compare 2 pathes
+ * Compare 2 paths
  * Paths could contain ".", "..", "//" in the path, thus normalize them first.
  * One or two of the paths could be a relative path.
  */
@@ -1186,9 +1186,19 @@ int
 slapd_comp_path(char *p0, char *p1)
 {
     int rval = 0;
-    char *norm_p0 = rel2abspath(p0);
-    char *norm_p1 = rel2abspath(p1);
+    char *norm_p0;
+    char *norm_p1;
 
+    /*
+     * Neither path should be NULL, but it's possible when bad thing happen.
+     * Return 0 which triggers an error in the caller
+     */
+    if (p0 == NULL || p1 == NULL){
+        return 0;
+    }
+
+    norm_p0 = rel2abspath(p0);
+    norm_p1 = rel2abspath(p1);
     rval = strcmp(norm_p0, norm_p1);
     slapi_ch_free_string(&norm_p0);
     slapi_ch_free_string(&norm_p1);
@@ -1579,3 +1589,18 @@ slapi_create_errormsg(
         va_end(ap);
     }
 }
+
+void
+get_internal_conn_op (uint64_t *connid, int32_t *op_id, int32_t *op_internal_id) {
+    struct slapi_td_log_op_state_t *op_state = slapi_td_get_log_op_state();
+
+    if (op_state != NULL) {
+        *connid = op_state->conn_id;
+        *op_id = op_state->op_id;
+        *op_internal_id = op_state->op_int_id;
+    } else {
+        *connid = 0;
+        *op_id = 0;
+        *op_internal_id = 0;
+    }
+}
