@@ -21,10 +21,8 @@ from lib389.utils import ensure_str, ensure_bytes, ensure_list_bytes
 
 MAJOR, MINOR, _, _, _ = sys.version_info
 
-logging.basicConfig(level=logging.DEBUG)
 log = logging.getLogger(__name__)
 
-
 class FormatDict(cidict):
     def __getitem__(self, name):
         if name in self:
@@ -503,7 +501,7 @@ class EntryAci(object):
                 rawaci += ('(%s);' % self.acidata["%s_raw_bindrules" %
                                                   key][0]['values'][-1])
         rawaci += ")"
-        return rawaci
+        return ensure_bytes(rawaci)
 
     def _find_terms(self, aci):
         if self.verbose:
