@@ -10,10 +10,11 @@ import ldap
 import ldap.dn
 from ldap import filter as ldap_filter
 import logging
+import json
 from functools import partial
 
 from lib389._entry import Entry
-from lib389._constants import DIRSRV_STATE_ONLINE
+from lib389._constants import DIRSRV_STATE_ONLINE, SER_ROOT_DN, SER_ROOT_PW
 from lib389.utils import (
         ensure_bytes, ensure_str, ensure_int, ensure_list_bytes, ensure_list_str,
         ensure_list_int
@@ -82,16 +83,14 @@ class DSLogging(object):
 class DSLdapObject(DSLogging):
     """A single instance of DSLdapObjects
 
-    :param instance: A instance
+    :param instance: An instance
     :type instance: lib389.DirSrv
     :param dn: Entry DN
     :type dn: str
-    :param batch: Not implemented
-    :type batch: bool
     """
 
     # TODO: Automatically create objects when they are requested to have properties added
-    def __init__(self, instance, dn=None, batch=False):
+    def __init__(self, instance, dn=None):
         self._instance = instance
         super(DSLdapObject, self).__init__(self._instance.verbose)
         # This allows some factor objects to be overriden
@@ -99,7 +98,6 @@ class DSLdapObject(DSLogging):
         if dn is not None:
             self._dn = ensure_str(dn)
 
-        self._batch = batch
         self._protected = True
         # Used in creation
         self._create_objectclasses = []
@@ -110,6 +108,7 @@ class DSLdapObject(DSLogging):
         self._lint_functions = None
         self._server_controls = None
         self._client_controls = None
+        self._object_filter = '(objectClass=*)'
 
     def __unicode__(self):
         val = self._dn
@@ -127,7 +126,7 @@ class DSLdapObject(DSLogging):
         :returns: Entry object
         """
 
-        return self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, attrlist=["*"], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
+        return self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=["*"], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
 
     def exists(self):
         """Check if the entry exists
@@ -136,7 +135,7 @@ class DSLdapObject(DSLogging):
         """
 
         try:
-            self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, attrsonly=1, serverctrls=self._server_controls, clientctrls=self._client_controls)
+            self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrsonly=1, serverctrls=self._server_controls, clientctrls=self._client_controls)
         except ldap.NO_SUCH_OBJECT:
             return False
 
@@ -148,7 +147,7 @@ class DSLdapObject(DSLogging):
         :returns: LDIF formatted string
         """
 
-        e = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, attrlist=["*"], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
+        e = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=["*"], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
         return e.__repr__()
 
     def display_attr(self, attr):
@@ -164,14 +163,15 @@ class DSLdapObject(DSLogging):
 
     def _jsonify(self, fn, *args, **kwargs):
         # This needs to map all the values to ensure_str
-        attrs = fn(*args, **kwargs)
+        attrs = fn(use_json=True, *args, **kwargs)
         str_attrs = {}
         for k in attrs:
             str_attrs[ensure_str(k)] = ensure_list_str(attrs[k])
 
-        response = { "dn": ensure_str(self._dn), "attrs" : str_attrs }
-        print('json response')
-        print(response)
+        # ensure all the keys are lowercase
+        str_attrs = dict((k.lower(), v) for k, v in list(str_attrs.items()))
+
+        response = json.dumps({"type": "entry", "dn": ensure_str(self._dn), "attrs": str_attrs})
 
         return response
 
@@ -186,7 +186,7 @@ class DSLdapObject(DSLogging):
 
         if (name.endswith('_json')):
             int_name = name.replace('_json', '')
-            pfunc = partial(self._jsonify, fn=getattr(self, int_name))
+            pfunc = partial(self._jsonify, getattr(self, int_name))
             return pfunc
 
     # We make this a property so that we can over-ride dynamically if needed
@@ -224,11 +224,16 @@ class DSLdapObject(DSLogging):
             raise ValueError("Invalid state. Cannot get presence on instance that is not ONLINE")
         self._log.debug("%s present(%r) %s" % (self._dn, attr, value))
 
-        e = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, attrlist=[attr, ], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
+        e = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=[attr, ], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
+        values = self.get_attr_vals_bytes(attr)
+        self._log.debug("%s contains %s" % (self._dn, values))
+
         if value is None:
-            return e.hasAttr(attr)
+            # We are just checking if SOMETHING is present ....
+            return len(values) > 0
         else:
-            return e.hasValue(attr, value)
+            # Check if a value really does exist.
+            return ensure_bytes(value).lower() in [x.lower() for x in values]
 
     def add(self, key, value):
         """Add an attribute with a value
@@ -252,6 +257,21 @@ class DSLdapObject(DSLogging):
         """
         self.set(key, value, action=ldap.MOD_REPLACE)
 
+    def replace_many(self, *args):
+        """Replace many key, value pairs in a single operation.
+        This is useful for configuration changes that require
+        atomic operation, and ease of use.
+
+        An example of usage is replace_many((key, value), (key, value))
+
+        No wrapping list is needed for the arguments.
+
+        :param *args: tuples of key,value to replace.
+        :type *args: (str, str)
+        """
+        mods = list(map(lambda t: (ldap.MOD_REPLACE, ensure_str(t[0]), ensure_bytes(t[1])), args))
+        return self._instance.modify_ext_s(self._dn, mods, serverctrls=self._server_controls, clientctrls=self._client_controls)
+
     # This needs to work on key + val, and key
     def remove(self, key, value):
         """Remove a value defined by key
@@ -275,8 +295,22 @@ class DSLdapObject(DSLogging):
         :type key: str
         """
 
-        for val in self.get_attr_vals(key):
-            self.remove(key, val)
+        try:
+            self.set(key, None, action=ldap.MOD_DELETE)
+        except ldap.NO_SUCH_ATTRIBUTE:
+            pass
+
+    def ensure_present(self, attr, value):
+        """Ensure that an attribute and value are present in a state,
+        or add it.
+
+        :param key: an attribute name
+        :type key: str
+        :param value: an attribute value
+        :type value: str
+        """
+        if not self.present(attr, value):
+            self.add(attr, value)
 
     # maybe this could be renamed?
     def set(self, key, value, action=ldap.MOD_REPLACE):
@@ -295,7 +329,20 @@ class DSLdapObject(DSLogging):
         :raises: ValueError - if instance is not online
         """
 
-        self._log.debug("%s set(%r, %r)" % (self._dn, key, value))
+        if action == ldap.MOD_ADD:
+            action_txt = "ADD"
+        elif action == ldap.MOD_REPLACE:
+            action_txt = "REPLACE"
+        elif action == ldap.MOD_DELETE:
+            action_txt = "DELETE"
+        else:
+            # This should never happen (bug!)
+            action_txt = "UNKNOWN"
+
+        if value is None or len(value) < 512:
+            self._log.debug("%s set %s: (%r, %r)" % (self._dn, action_txt, key, value))
+        else:
+            self._log.debug("%s set %s: (%r, value too large)" % (self._dn, action_txt, key))
         if self._instance.state != DIRSRV_STATE_ONLINE:
             raise ValueError("Invalid state. Cannot set properties on instance that is not ONLINE.")
 
@@ -305,10 +352,8 @@ class DSLdapObject(DSLogging):
         elif value is not None:
             value = [ensure_bytes(value)]
 
-        if self._batch:
-            pass
-        else:
-            return self._instance.modify_ext_s(self._dn, [(action, key, value)], serverctrls=self._server_controls, clientctrls=self._client_controls)
+        return self._instance.modify_ext_s(self._dn, [(action, key, value)],
+             serverctrls=self._server_controls, clientctrls=self._client_controls)
 
     def apply_mods(self, mods):
         """Perform modification operation using several mods at once
@@ -328,8 +373,8 @@ class DSLdapObject(DSLogging):
                 key, value = mod
             elif len(mod) == 3:
                 action, key, value = mod
-                if action != ldap.MOD_REPLACE or \
-                   action != ldap.MOD_ADD or \
+                if action != ldap.MOD_REPLACE and \
+                   action != ldap.MOD_ADD and \
                    action != ldap.MOD_DELETE:
                     raise ValueError('Invalid mod action(%s)' % str(action))
             else:
@@ -386,7 +431,7 @@ class DSLdapObject(DSLogging):
                 return False
         return True
 
-    def get_compare_attrs(self):
+    def get_compare_attrs(self, use_json=False):
         """Get a dictionary having attributes to be compared
         i.e. excluding self._compare_exclude
         """
@@ -398,7 +443,7 @@ class DSLdapObject(DSLogging):
         compare_attrs_dict = {attr:all_attrs_dict[attr] for attr in compare_attrs}
         return compare_attrs_dict
 
-    def get_all_attrs(self):
+    def get_all_attrs(self, use_json=False):
         """Get a dictionary having all the attributes of the entry
 
         :returns: Dict with real attributes and operational attributes
@@ -409,20 +454,20 @@ class DSLdapObject(DSLogging):
             raise ValueError("Invalid state. Cannot get properties on instance that is not ONLINE")
         else:
             # retrieving real(*) and operational attributes(+)
-            attrs_entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, attrlist=["*", "+"], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
+            attrs_entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=["*", "+"], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
             # getting dict from 'entry' object
             attrs_dict = attrs_entry.data
             return attrs_dict
 
-    def get_attrs_vals(self, keys):
+    def get_attrs_vals(self, keys, use_json=False):
         self._log.debug("%s get_attrs_vals(%r)" % (self._dn, keys))
         if self._instance.state != DIRSRV_STATE_ONLINE:
             raise ValueError("Invalid state. Cannot get properties on instance that is not ONLINE")
         else:
-            entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, attrlist=keys, serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
+            entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=keys, serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
             return entry.getValuesSet(keys)
 
-    def get_attr_vals(self, key):
+    def get_attr_vals(self, key, use_json=False):
         self._log.debug("%s get_attr_vals(%r)" % (self._dn, key))
         # We might need to add a state check for NONE dn.
         if self._instance.state != DIRSRV_STATE_ONLINE:
@@ -432,10 +477,17 @@ class DSLdapObject(DSLogging):
         else:
             # It would be good to prevent the entry code intercepting this ....
             # We have to do this in this method, because else we ignore the scope base.
-            entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, attrlist=[key], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
-            return entry.getValues(key)
+            entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=[key], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
+            vals = entry.getValues(key)
+            if use_json:
+                result = {key: []}
+                for val in vals:
+                    result[key].append(val)
+                return result
+            else:
+                return vals
 
-    def get_attr_val(self, key):
+    def get_attr_val(self, key, use_json=False):
         self._log.debug("%s getVal(%r)" % (self._dn, key))
         # We might need to add a state check for NONE dn.
         if self._instance.state != DIRSRV_STATE_ONLINE:
@@ -443,10 +495,10 @@ class DSLdapObject(DSLogging):
             # In the future, I plan to add a mode where if local == true, we
             # can use get on dse.ldif to get values offline.
         else:
-            entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, attrlist=[key], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
+            entry = self._instance.search_ext_s(self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=[key], serverctrls=self._server_controls, clientctrls=self._client_controls)[0]
             return entry.getValue(key)
 
-    def get_attr_val_bytes(self, key):
+    def get_attr_val_bytes(self, key, use_json=False):
         """Get a single attribute value from the entry in bytes type
 
         :param key: An attribute name
@@ -457,7 +509,7 @@ class DSLdapObject(DSLogging):
 
         return ensure_bytes(self.get_attr_val(key))
 
-    def get_attr_vals_bytes(self, key):
+    def get_attr_vals_bytes(self, key, use_json=False):
         """Get attribute values from the entry in bytes type
 
         :param key: An attribute name
@@ -468,7 +520,7 @@ class DSLdapObject(DSLogging):
 
         return ensure_list_bytes(self.get_attr_vals(key))
 
-    def get_attr_val_utf8(self, key):
+    def get_attr_val_utf8(self, key, use_json=False):
         """Get a single attribute value from the entry in utf8 type
 
         :param key: An attribute name
@@ -479,7 +531,22 @@ class DSLdapObject(DSLogging):
 
         return ensure_str(self.get_attr_val(key))
 
-    def get_attr_vals_utf8(self, key):
+    def get_attr_val_utf8_l(self, key, use_json=False):
+        """Get a single attribute value from the entry in utf8 type
+
+        :param key: An attribute name
+        :type key: str
+        :returns: A single bytes value
+        :raises: ValueError - if instance is offline
+        """
+
+        x = self.get_attr_val(key)
+        if x is not None:
+            return ensure_str(x).lower()
+        else:
+            return None
+
+    def get_attr_vals_utf8(self, key, use_json=False):
         """Get attribute values from the entry in utf8 type
 
         :param key: An attribute name
@@ -490,7 +557,18 @@ class DSLdapObject(DSLogging):
 
         return ensure_list_str(self.get_attr_vals(key))
 
-    def get_attr_val_int(self, key):
+    def get_attr_vals_utf8_l(self, key, use_json=False):
+        """Get attribute values from the entry in utf8 type and lowercase
+
+        :param key: An attribute name
+        :type key: str
+        :returns: A single bytes value
+        :raises: ValueError - if instance is offline
+        """
+
+        return [x.lower() for x in ensure_list_str(self.get_attr_vals(key))]
+
+    def get_attr_val_int(self, key, use_json=False):
         """Get a single attribute value from the entry in int type
 
         :param key: An attribute name
@@ -501,7 +579,7 @@ class DSLdapObject(DSLogging):
 
         return ensure_int(self.get_attr_val(key))
 
-    def get_attr_vals_int(self, key):
+    def get_attr_vals_int(self, key, use_json=False):
         """Get attribute values from the entry in int type
 
         :param key: An attribute name
@@ -523,20 +601,41 @@ class DSLdapObject(DSLogging):
     def set_values(self, values, action=ldap.MOD_REPLACE):
         pass
 
-    # If the account can be bound to, this will attempt to do so. We don't check
-    # for exceptions, just pass them back!
-    def bind(self, password=None, *args, **kwargs):
-        """Open a new connection and bind with the entry.
-        You can pass arguments that will be passed to openConnection.
+    # Modifies the DN of an entry to the new fqdn provided
+    def rename(self, new_rdn, newsuperior=None):
+        """Renames the object within the tree.
 
-        :param password: An entry password
-        :type password: str
-        :returns: Connection with a binding as the entry
+        If you provide a newsuperior, this will move the object in the tree.
+        If you only provide a new_rdn, it stays in the same branch, but just
+        changes the rdn.
+
+        Note, if you use newsuperior, you may move this object outside of the
+        scope of the related DSLdapObjects manager, which may cause it not to
+        appear in .get() requests.
+
+        :param new_rdn: RDN of the new entry
+        :type new_rdn: str
+        :param newsuperior: New parent DN
+        :type newsuperior: str
         """
+        # When we are finished with this, we need to update our DN
+        # To do this, we probably need to search the new rdn as a filter,
+        # and the superior as the base (if it changed)
+        if self._protected:
+            return
+        self._instance.rename_s(self._dn, new_rdn, newsuperior, serverctrls=self._server_controls, clientctrls=self._client_controls)
+        search_base = self._basedn
+        if newsuperior != None:
+            # Well, the new DN should be rdn + newsuperior.
+            self._dn = '%s,%s' % (new_rdn, newsuperior)
+        else:
+            old_dn_parts = ldap.explode_dn(self._dn)
+            # Replace the rdn
+            old_dn_parts[0] = new_rdn
+            self._dn = ",".join(old_dn_parts)
+        assert self.exists()
 
-        conn = self._instance.openConnection(*args, **kwargs)
-        conn.simple_bind_s(self.dn, password)
-        return conn
+        # assert we actually got the change right ....
 
     def delete(self):
         """Deletes the object defined by self._dn.
@@ -567,9 +666,10 @@ class DSLdapObject(DSLogging):
 
         # I think this needs to be made case insensitive
         # How will this work with the dictionary?
-        for attr in self._must_attributes:
-            if properties.get(attr, None) is None:
-                raise ldap.UNWILLING_TO_PERFORM('Attribute %s must not be None' % attr)
+        if self._must_attributes is not None:
+            for attr in self._must_attributes:
+                if properties.get(attr, None) is None:
+                    raise ldap.UNWILLING_TO_PERFORM('Attribute %s must not be None' % attr)
 
         # Make sure the naming attribute is present
         if properties.get(self._rdn_attribute, None) is None and rdn is None:
@@ -582,7 +682,7 @@ class DSLdapObject(DSLogging):
                 pass
             else:
                 # Turn it into a list instead.
-                properties[k] = [v,]
+                properties[k] = [v, ]
 
         # This change here, means we can pre-load a full dn to _dn, or we can
         # accept based on the rdn
@@ -592,7 +692,9 @@ class DSLdapObject(DSLogging):
             if basedn is None:
                 raise ldap.UNWILLING_TO_PERFORM('Invalid request to create. basedn cannot be None')
 
-            if properties.get(self._rdn_attribute, None) is not None:
+            if rdn is not None:
+                tdn = ensure_str('%s,%s' % (rdn, basedn))
+            elif properties.get(self._rdn_attribute, None) is not None:
                 # Favour the value in the properties dictionary
                 v = properties.get(self._rdn_attribute)
                 rdn = ensure_str(v[0])
@@ -613,6 +715,51 @@ class DSLdapObject(DSLogging):
         # Do we need to do extra dn validation here?
         return (tdn, str_props)
 
+    def _create(self, rdn=None, properties=None, basedn=None, ensure=False):
+        """Internal implementation of create. This is used by ensure
+        and create, to prevent code duplication. You should *never* call
+        this method directly.
+        """
+        assert(len(self._create_objectclasses) > 0)
+        basedn = ensure_str(basedn)
+        self._log.debug('Checking "%s" under %s : %s' % (rdn, basedn, properties))
+        # Add the objectClasses to the properties
+        (dn, valid_props) = self._validate(rdn, properties, basedn)
+        # Check if the entry exists or not? .add_s is going to error anyway ...
+        self._log.debug('Validated dn %s : valid_props %s' % (dn, valid_props))
+
+        exists = False
+
+        try:
+            self._instance.search_ext_s(dn, ldap.SCOPE_BASE, self._object_filter, attrsonly=1, serverctrls=self._server_controls, clientctrls=self._client_controls)
+            exists = True
+        except ldap.NO_SUCH_OBJECT:
+            pass
+
+        if exists and ensure:
+            # update properties
+            self._log.debug('Exists %s' % dn)
+            self._dn = dn
+            # Now use replace_many to setup our values
+            mods = []
+            for k,v in valid_props.items():
+                mods.append( (ldap.MOD_REPLACE, k, v))
+            self._instance.modify_ext_s(self._dn, mods, serverctrls=self._server_controls, clientctrls=self._client_controls)
+        elif exists and not ensure:
+            # raise "already exists."
+            raise ldap.ALREADY_EXISTS("Entry %s already exists" % dn)
+        if not exists:
+            self._log.debug('Creating %s' % dn)
+            e = Entry(dn)
+            e.update({'objectclass': ensure_list_bytes(self._create_objectclasses)})
+            e.update(valid_props)
+            # We rely on exceptions here to indicate failure to the parent.
+            self._log.debug('Creating entry %s : %s' % (dn, e))
+            self._instance.add_ext_s(e, serverctrls=self._server_controls, clientctrls=self._client_controls)
+            # If it worked, we need to fix our instance dn
+            self._dn = dn
+        return self
+
     def create(self, rdn=None, properties=None, basedn=None):
         """Add a new entry
 
@@ -625,24 +772,22 @@ class DSLdapObject(DSLogging):
 
         :returns: DSLdapObject of the created entry
         """
+        return self._create(rdn, properties, basedn, ensure=False)
 
-        assert(len(self._create_objectclasses) > 0)
-        basedn = ensure_str(basedn)
-        self._log.debug('Creating "%s" under %s : %s' % (rdn, basedn, properties))
-        # Add the objectClasses to the properties
-        (dn, valid_props) = self._validate(rdn, properties, basedn)
-        # Check if the entry exists or not? .add_s is going to error anyway ...
-        self._log.debug('Validated dn %s : valid_props %s' % (dn, valid_props))
+    def ensure_state(self, rdn=None, properties=None, basedn=None):
+        """Ensure an entry exists with the following state, created
+        if necessary.
 
-        e = Entry(dn)
-        e.update({'objectclass': ensure_list_bytes(self._create_objectclasses)})
-        e.update(valid_props)
-        # We rely on exceptions here to indicate failure to the parent.
-        self._log.debug('Creating entry %s : %s' % (dn, e))
-        self._instance.add_ext_s(e, serverctrls=self._server_controls, clientctrls=self._client_controls)
-        # If it worked, we need to fix our instance dn
-        self._dn = dn
-        return self
+        :param rdn: RDN of the new entry
+        :type rdn: str
+        :param properties: Attributes for the new entry
+        :type properties: dict
+        :param basedn: Base DN of the new entry
+        :type rdn: str
+
+        :returns: DSLdapObject of the created entry
+        """
+        return self._create(rdn, properties, basedn, ensure=True)
 
     def lint(self):
         """Override this to create a linter for a type. This means that we can detect
@@ -672,20 +817,19 @@ class DSLdapObject(DSLogging):
                 results.append(result)
         return results
 
-# A challenge of this, is how do we manage indexes? They have two naming attribunes....
+
+# A challenge of this, is how do we manage indexes? They have two naming attributes....
 
 class DSLdapObjects(DSLogging):
     """The object represents the next idea: "Everything is an instance of something
     that exists in this way", i.e. we unite LDAP entries by some
     set of parameters with the object.
 
-    :param instance: A instance
+    :param instance: An instance
     :type instance: lib389.DirSrv
-    :param batch: Not implemented
-    :type batch: bool
     """
 
-    def __init__(self, instance, batch=False):
+    def __init__(self, instance):
         self._childobject = DSLdapObject
         self._instance = instance
         super(DSLdapObjects, self).__init__(self._instance.verbose)
@@ -694,7 +838,6 @@ class DSLdapObjects(DSLogging):
         self._list_attrlist = ['dn']
         # Copy this from the child if we need.
         self._basedn = ""
-        self._batch = batch
         self._scope = ldap.SCOPE_SUBTREE
         self._server_controls = None
         self._client_controls = None
@@ -709,7 +852,7 @@ class DSLdapObjects(DSLogging):
         # have "many" possible child types, this allows us to overload
         # and select / return the right one through ALL our get/list/create
         # functions with very little work on the behalf of the overloader
-        return self._childobject(instance=self._instance, dn=dn, batch=self._batch)
+        return self._childobject(instance=self._instance, dn=dn)
 
     def list(self):
         """Get a list of children entries (DSLdapObject, Replica, etc.) using a base DN
@@ -731,20 +874,22 @@ class DSLdapObjects(DSLogging):
                 attrlist=self._list_attrlist,
                 serverctrls=self._server_controls, clientctrls=self._client_controls
             )
-            # def __init__(self, instance, dn=None, batch=False):
+            # def __init__(self, instance, dn=None):
             insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
         except ldap.NO_SUCH_OBJECT:
             # There are no objects to select from, se we return an empty array
             insts = []
         return insts
 
-    def get(self, selector=[], dn=None):
+    def get(self, selector=[], dn=None, json=False):
         """Get a child entry (DSLdapObject, Replica, etc.) with dn or selector
         using a base DN and objectClasses of our object (DSLdapObjects, Replicas, etc.)
 
+        Note that * is not a valid selector, you should use "list()" instead.
+
         :param dn: DN of wanted entry
         :type dn: str
-        :param selector: An additional filter to objectClasses, i.e. 'backend_name'
+        :param selector: An additional filter to search for, i.e. 'backend_name'. The attributes selected are based on object type, ie user will search for uid and cn.
         :type dn: str
 
         :returns: A child entry
@@ -760,12 +905,16 @@ class DSLdapObjects(DSLogging):
             raise ldap.NO_SUCH_OBJECT("No object exists given the filter criteria %s" % selector)
         if len(results) > 1:
             raise ldap.UNWILLING_TO_PERFORM("Too many objects matched selection criteria %s" % selector)
-        return self._entry_to_instance(results[0].dn, results[0])
+        if json:
+            return self._entry_to_instance(results[0].dn, results[0]).get_all_attrs_json()
+        else:
+            return self._entry_to_instance(results[0].dn, results[0])
 
     def _get_dn(self, dn):
         # This will yield and & filter for objectClass with as many terms as needed.
         filterstr = self._get_objectclass_filter()
         self._log.debug('_gen_dn filter = %s' % filterstr)
+        self._log.debug('_gen_dn dn = %s' % dn)
         return self._instance.search_ext_s(
             base=dn,
             scope=ldap.SCOPE_BASE,
@@ -803,19 +952,6 @@ class DSLdapObjects(DSLogging):
         if type(properties) != dict:
             raise ldap.UNWILLING_TO_PERFORM("properties must be a dictionary")
 
-        # Get the rdn out of the properties if it's unset???
-        if rdn is None and self._rdn_attribute in properties:
-            # First see if we can get it from the properties.
-            trdn = properties.get(self._rdn_attribute)
-            if type(trdn) == str:
-                rdn = trdn
-            elif type(trdn) == list and len(trdn) != 1:
-                raise ldap.UNWILLING_TO_PERFORM("Cannot determine rdn %s from properties. Too many choices" % (self._rdn_attribute))
-            elif type(trdn) == list:
-                rdn = trdn[0]
-            else:
-                raise ldap.UNWILLING_TO_PERFORM("Cannot determine rdn %s from properties, Invalid type" % type(trdn))
-
         return (rdn, properties)
 
     def create(self, rdn=None, properties=None):
@@ -833,8 +969,32 @@ class DSLdapObjects(DSLogging):
         # This may not work in all cases, especially when we consider plugins.
         #
         co = self._entry_to_instance(dn=None, entry=None)
-        # Make the rdn naming attr avaliable
+        # Make the rdn naming attr available
         self._rdn_attribute = co._rdn_attribute
         (rdn, properties) = self._validate(rdn, properties)
         # Now actually commit the creation req
         return co.create(rdn, properties, self._basedn)
+
+    def ensure_state(self, rdn=None, properties=None):
+        """Create an object under base DN of our entry, or
+        assert it exists and update it's properties.
+
+        :param rdn: RDN of the new entry
+        :type rdn: str
+        :param properties: Attributes for the new entry
+        :type properties: dict
+
+        :returns: DSLdapObject of the created entry
+        """
+
+        # Should we inject the rdn to properties?
+        # This may not work in all cases, especially when we consider plugins.
+        #
+        co = self._entry_to_instance(dn=None, entry=None)
+        # Make the rdn naming attr avaliable
+        self._rdn_attribute = co._rdn_attribute
+        (rdn, properties) = self._validate(rdn, properties)
+        # Now actually commit the creation req
+        return co.ensure_state(rdn, properties, self._basedn)
+
+
