@@ -6,6 +6,7 @@
 # See LICENSE for details.
 # --- END COPYRIGHT BLOCK ---
 
+from datetime import datetime
 import ldap
 from lib389._constants import *
 from lib389.properties import *
@@ -21,6 +22,8 @@ from lib389.exceptions import NoSuchEntryError, InvalidArgumentError
 # We need to be a factor to the backend monitor
 from lib389.monitor import MonitorBackend
 from lib389.index import Indexes
+from lib389.tasks import ImportTask, ExportTask
+from lib389.encrypted_attributes import EncryptedAttrs
 
 # This is for sample entry creation.
 from lib389.configurations import get_sample_entries
@@ -64,11 +67,11 @@ class BackendLegacy(object):
 
         filt = "(objectclass=%s)" % BACKEND_OBJECTCLASS_VALUE
         if backend_dn:
-            self.log.info("List backend %s" % backend_dn)
+            self.log.info("List backend %s", backend_dn)
             base = backend_dn
             scope = ldap.SCOPE_BASE
         elif suffix:
-            self.log.info("List backend with suffix=%s" % suffix)
+            self.log.info("List backend with suffix=%s", suffix)
             base = DN_PLUGIN
             scope = ldap.SCOPE_SUBTREE
             filt = ("(&%s(|(%s=%s)(%s=%s)))" %
@@ -78,7 +81,7 @@ class BackendLegacy(object):
                      normalizeDN(suffix))
                     )
         elif bename:
-            self.log.info("List backend 'cn=%s'" % bename)
+            self.log.info("List backend 'cn=%s'", bename)
             base = "%s=%s,%s" % (BACKEND_PROPNAME_TO_ATTRNAME[BACKEND_NAME],
                                  bename, DN_LDBM)
             scope = ldap.SCOPE_BASE
@@ -155,7 +158,7 @@ class BackendLegacy(object):
                 (suffix, backend_dn, bename))
         elif len(be_ents) > 1:
             for ent in be_ents:
-                self.log.fatal("Multiple backend match the definition: %s" %
+                self.log.fatal("Multiple backend match the definition: %s",
                                ent.dn)
             if (not suffix) and (not backend_dn) and (not bename):
                 raise ldap.UNWILLING_TO_PERFORM(
@@ -193,10 +196,10 @@ class BackendLegacy(object):
         # finally delete the backend children and the backend itself
         ents = self.conn.search_s(be_ent.dn, ldap.SCOPE_ONELEVEL)
         for ent in ents:
-            self.log.debug("Delete entry children %s" % (ent.dn))
+            self.log.debug("Delete entry children %s", ent.dn)
             self.conn.delete_s(ent.dn)
 
-        self.log.debug("Delete backend entry %s" % (be_ent.dn))
+        self.log.debug("Delete backend entry %s", be_ent.dn)
         self.conn.delete_s(be_ent.dn)
 
         return
@@ -246,12 +249,12 @@ class BackendLegacy(object):
                         (BACKEND_PROPNAME_TO_ATTRNAME[BACKEND_NAME],
                          bename, parent))
                 filt = "(objectclass=%s)" % BACKEND_OBJECTCLASS_VALUE
-                self.log.debug("_getBackendName: baser=%s : fileter=%s" %
-                               (base, filt))
+                self.log.debug("_getBackendName: baser=%s : fileter=%s",
+                               base, filt)
                 try:
                     self.conn.getEntry(base, ldap.SCOPE_BASE, filt)
                 except (NoSuchEntryError, ldap.NO_SUCH_OBJECT):
-                    self.log.info("backend name will be %s" % bename)
+                    self.log.info("backend name will be %s", bename)
                     return bename
                 index += 1
 
@@ -265,7 +268,7 @@ class BackendLegacy(object):
 
         # Check it does not already exist a backend for that suffix
         if self.conn.verbose:
-            self.log.info("Checking suffix %s for existence" % suffix)
+            self.log.info("Checking suffix %s for existence", suffix)
         ents = self.conn.backend.list(suffix=suffix)
         if len(ents) != 0:
             raise ldap.ALREADY_EXISTS
@@ -317,13 +320,13 @@ class BackendLegacy(object):
                      properties[BACKEND_CHAIN_BIND_PW]
                      })
 
-            self.log.debug("adding entry: %r" % entry)
+            self.log.debug("adding entry: %r", entry)
             self.conn.add_s(entry)
         except ldap.ALREADY_EXISTS as e:
-            self.log.error("Entry already exists: %r" % dn)
+            self.log.error("Entry already exists: %r", dn)
             raise ldap.ALREADY_EXISTS("%s : %r" % (e, dn))
         except ldap.LDAPError as e:
-            self.log.error("Could not add backend entry: %r" % dn)
+            self.log.error("Could not add backend entry: %r", dn)
             raise e
 
         backend_entry = self.conn._test_entry(dn, ldap.SCOPE_BASE)
@@ -365,7 +368,7 @@ class BackendLegacy(object):
 
             attrs = [attr_suffix]
             ent = self.conn.getEntry(name, ldap.SCOPE_BASE, filt, attrs)
-            self.log.debug("toSuffix: %s found by its DN" % ent.dn)
+            self.log.debug("toSuffix: %s found by its DN", ent.dn)
 
             if not ent.hasValue(attr_suffix):
                 raise ValueError("Entry has no %s attribute %r" %
@@ -387,21 +390,19 @@ class BackendLegacy(object):
 
 class Backend(DSLdapObject):
     """Backend DSLdapObject with:
-    - must attributes = ['cn, 'nsslapd-suffix']
+    - must attributes = ['cn', 'nsslapd-suffix']
     - RDN attribute is 'cn'
 
-    :param instance: A instance
+    :param instance: An instance
     :type instance: lib389.DirSrv
     :param dn: Entry DN
     :type dn: str
-    :param batch: Not implemented
-    :type batch: bool
     """
 
     _must_attributes = ['nsslapd-suffix', 'cn']
 
-    def __init__(self, instance, dn=None, batch=False):
-        super(Backend, self).__init__(instance, dn, batch)
+    def __init__(self, instance, dn=None):
+        super(Backend, self).__init__(instance, dn)
         self._rdn_attribute = 'cn'
         self._must_attributes = ['nsslapd-suffix', 'cn']
         self._create_objectclasses = ['top', 'extensibleObject', BACKEND_OBJECTCLASS_VALUE]
@@ -518,7 +519,10 @@ class Backend(DSLdapObject):
         self._instance.index.delete_all(bename)
 
         # Now remove our children, this is all ldbm config
-        self._instance.delete_branch_s(self._dn, ldap.SCOPE_ONELEVEL)
+
+        configs = self._instance.search_s(self._dn, ldap.SCOPE_ONELEVEL)
+        for c in configs:
+            self._instance.delete_branch_s(c.dn, ldap.SCOPE_SUBTREE)
         # The super will actually delete ourselves.
         super(Backend, self).delete()
 
@@ -543,10 +547,14 @@ class Backend(DSLdapObject):
             return result
         return None
 
+    def get_mapping_tree(self):
+        suffix = self.get_attr_val_utf8('nsslapd-suffix')
+        return self._mts.get(suffix)
+
     def get_monitor(self):
         """Get a MonitorBackend(DSLdapObject) for the backend"""
 
-        monitor = MonitorBackend(instance=self._instance, dn= "cn=monitor,%s" % self._dn, batch=self._batch)
+        monitor = MonitorBackend(instance=self._instance, dn= "cn=monitor,%s" % self._dn)
         return monitor
 
     def get_indexes(self):
@@ -556,21 +564,123 @@ class Backend(DSLdapObject):
         return indexes
     # Future: add reindex task for this be.
 
+    def import_ldif(self, ldifs, chunk_size=None, encrypted=False, gen_uniq_id=False, only_core=False,
+                    include_suffixes=None, exclude_suffixes=None):
+        """Do an import of the suffix"""
+
+        bs = Backends(self._instance)
+        task = bs.import_ldif(self.rdn, ldifs, chunk_size, encrypted, gen_uniq_id, only_core,
+                              include_suffixes, exclude_suffixes)
+        return task
+
+    def export_ldif(self, ldif=None, use_id2entry=False, encrypted=False, min_base64=False, no_uniq_id=False,
+                    replication=False, not_folded=False, no_seq_num=False, include_suffixes=None, exclude_suffixes=None):
+        """Do an export of the suffix"""
+
+        bs = Backends(self._instance)
+        task = bs.export_ldif(self.rdn, ldif, use_id2entry, encrypted, min_base64, no_uniq_id,
+                              replication, not_folded, no_seq_num, include_suffixes, exclude_suffixes)
+        return task
+
+    def get_encrypted_attrs(self):
+        """Get Encrypted Attributes(DSLdapObject) for the backend"""
+
+        encrypt_attrs = EncryptedAttrs(instance=self._instance, basedn="cn=encrypted attributes,%s" % self._dn)
+        return encrypt_attrs
+
 
 class Backends(DSLdapObjects):
     """DSLdapObjects that represents DN_LDBM base DN
     This only does ldbm backends. Chaining backends are a special case
     of this, so they can be subclassed off.
 
-    :param instance: A instance
+    :param instance: An instance
     :type instance: lib389.DirSrv
-    :param batch: Not implemented
-    :type batch: bool
     """
 
-    def __init__(self, instance, batch=False):
-        super(Backends, self).__init__(instance=instance, batch=batch)
+    def __init__(self, instance, basedn=None):
+        # Basedn has to be here, despite not being used to satisfy
+        # cli_base _generic_create.
+        super(Backends, self).__init__(instance=instance)
         self._objectclasses = [BACKEND_OBJECTCLASS_VALUE]
         self._filterattrs = ['cn', 'nsslapd-suffix', 'nsslapd-directory']
         self._childobject = Backend
         self._basedn = DN_LDBM
+
+    def import_ldif(self, be_name, ldifs, chunk_size=None, encrypted=False, gen_uniq_id=None, only_core=False,
+                    include_suffixes=None, exclude_suffixes=None):
+        """Do an import of the suffix"""
+
+        if not ldifs:
+            self.log.error("import_ldif: LDIF filename is missing")
+            return False
+        ldif_paths = []
+        for ldif in list(ldifs):
+            if not ldif.startswith("/"):
+                if ldif.endswith(".ldif"):
+                    ldif = os.path.join(self._instance.ds_paths.ldif_dir, ldif)
+                else:
+                    ldif = os.path.join(self._instance.ds_paths.ldif_dir, "%s.ldif" % ldif)
+            ldif_paths.append(ldif)
+
+        task = ImportTask(self._instance)
+        task_properties = {'nsInstance': be_name,
+                           'nsFilename': ldif_paths}
+        if include_suffixes is not None:
+            task_properties['nsIncludeSuffix'] = include_suffixes
+        if exclude_suffixes is not None:
+            task_properties['nsExcludeSuffix'] = exclude_suffixes
+        if encrypted:
+            task_properties['nsExportDecrypt'] = 'true'
+        if only_core:
+            task_properties['nsImportIndexAttrs'] = 'false'
+        if chunk_size is not None:
+            task_properties['nsImportChunkSize'] = chunk_size
+        if gen_uniq_id is not None:
+            if gen_uniq_id in ("none", "empty") or gen_uniq_id.startswith("deterministic"):
+                raise ValueError("'gen_uniq_id should be none (no unique ID) |"
+                                 "empty (time-based ID) | deterministic namespace (name-based ID)")
+            task_properties['nsUniqueIdGenerator'] = gen_uniq_id
+
+        task.create(properties=task_properties)
+
+        return task
+
+    def export_ldif(self, be_names, ldif=None, use_id2entry=False, encrypted=False, min_base64=False, no_dump_uniq_id=False,
+                    replication=False, not_folded=False, no_seq_num=False, include_suffixes=None, exclude_suffixes=None):
+        """Do an export of the suffix"""
+
+        task = ExportTask(self._instance)
+        task_properties = {'nsInstance': be_names}
+        if ldif is not None and not ldif.startswith("/"):
+            if ldif.endswith(".ldif"):
+                task_properties['nsFilename'] = os.path.join(self._instance.ds_paths.ldif_dir, ldif)
+            else:
+                task_properties['nsFilename'] = os.path.join(self._instance.ds_paths.ldif_dir, "%s.ldif" % ldif)
+        else:
+            tnow = datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
+            task_properties['nsFilename'] = os.path.join(self._instance.ds_paths.ldif_dir,
+                                                         "%s-%s-%s.ldif" % (self._instance.serverid,
+                                                                            "-".join(be_names), tnow))
+        if include_suffixes is not None:
+            task_properties['nsIncludeSuffix'] = include_suffixes
+        if exclude_suffixes is not None:
+            task_properties['nsExcludeSuffix'] = exclude_suffixes
+        if use_id2entry:
+            task_properties['nsUseId2Entry'] = 'true'
+        if encrypted:
+            task_properties['nsExportDecrypt'] = 'true'
+        if replication:
+            task_properties['nsExportReplica'] = 'true'
+        if min_base64:
+            task_properties['nsMinimalEncoding'] = 'true'
+        if not_folded:
+            task_properties['nsNoWrap'] = 'true'
+        if no_dump_uniq_id:
+            task_properties['nsDumpUniqId'] = 'false'
+        if no_seq_num:
+            task_properties['nsPrintKey'] = 'false'
+
+        task = task.create(properties=task_properties)
+        return task
+
