@@ -15,11 +15,7 @@ from datetime import datetime
 from lib389 import Entry
 from lib389._mapped_object import DSLdapObject
 from lib389.exceptions import Error
-from lib389._constants import (
-        DEFAULT_SUFFIX, DEFAULT_BENAME, DN_EXPORT_TASK, DN_BACKUP_TASK,
-        DN_IMPORT_TASK, DN_RESTORE_TASK, DN_INDEX_TASK, DN_MBO_TASK,
-        DN_TOMB_FIXUP_TASK, DN_TASKS, DIRSRV_STATE_ONLINE
-        )
+from lib389._constants import *
 from lib389.properties import (
         TASK_WAIT, EXPORT_REPL_INFO, MT_PROPNAME_TO_ATTRNAME, MT_SUFFIX,
         TASK_TOMB_STRIP
@@ -27,8 +23,16 @@ from lib389.properties import (
 
 
 class Task(DSLdapObject):
-    def __init__(self, instance, dn=None, batch=False):
-        super(Task, self).__init__(instance, dn, batch)
+    """A single instance of a task entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn=None):
+        super(Task, self).__init__(instance, dn)
         self._rdn_attribute = 'cn'
         self._must_attributes = ['cn']
         self._create_objectclasses = ['top', 'extensibleObject']
@@ -37,6 +41,7 @@ class Task(DSLdapObject):
 
     def is_complete(self):
         """Return True if task is complete, else False."""
+
         self._exit_code = self.get_attr_val("nsTaskExitCode")
         if not self.exists() or self._exit_code is not None:
             return True
@@ -44,6 +49,7 @@ class Task(DSLdapObject):
 
     def get_exit_code(self):
         """Return task's exit code if task is complete, else None."""
+
         if self.is_complete():
             try:
                 return int(self._exit_code)
@@ -51,38 +57,96 @@ class Task(DSLdapObject):
                 return None
         return None
 
-    def wait(self):
+    def wait(self, timeout=120):
         """Wait until task is complete."""
-        while True:
+
+        count = 0
+        while count < timeout:
             if self.is_complete():
                 break
-            time.sleep(1)
+            count = count + 1
+            time.sleep(2)
+
+    def create(self, rdn=None, properties={}, basedn=None):
+        """Create a Task entry
+
+        :param rdn: RDN of the new entry
+        :type rdn: str
+        :param properties: Attributes for the new entry
+        :type properties: dict
+        :param basedn: Base DN of the new entry
+        :type rdn: str
+
+        :returns: DSLdapObject of the created entry
+        """
 
-    def create(self, rdn=None, properties=None, basedn=None):
         properties['cn'] = self.cn
         return super(Task, self).create(rdn, properties, basedn)
 
     @staticmethod
     def _get_task_date():
         """Return a timestamp to use in naming new task entries."""
-        return datetime.now().strftime('%Y_%m_%d_%H_%M_%S')
+
+        return datetime.now().isoformat()
+
+
+class AutomemberRebuildMembershipTask(Task):
+    """A single instance of automember rebuild membership task entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
+        self.cn = 'automember_rebuild_' + Task._get_task_date()
+        dn = "cn=" + self.cn + "," + DN_AUTOMEMBER_REBUILD_TASK
+
+        super(AutomemberRebuildMembershipTask, self).__init__(instance, dn)
+        self._must_attributes.extend(['basedn', 'filter'])
+
+
+class FixupLinkedAttributesTask(Task):
+    """A single instance of fixup linked attributes task entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
+        self.cn = 'fixup_linked_attrs_' + Task._get_task_date()
+        dn = "cn=" + self.cn + "," + DN_FIXUP_LINKED_ATTIBUTES
+
+        super(FixupLinkedAttributesTask, self).__init__(instance, dn)
+        self._must_attributes.extend(['basedn'])
 
 
 class MemberOfFixupTask(Task):
-    def __init__(self, instance, dn=None, batch=False):
+    """A single instance of memberOf task entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
         self.cn = 'memberOf_fixup_' + Task._get_task_date()
         dn = "cn=" + self.cn + "," + DN_MBO_TASK
 
-        super(MemberOfFixupTask, self).__init__(instance, dn, batch)
+        super(MemberOfFixupTask, self).__init__(instance, dn)
         self._must_attributes.extend(['basedn'])
 
 
 class USNTombstoneCleanupTask(Task):
-    def __init__(self, instance, dn=None, batch=False):
+    """A single instance of USN tombstone cleanup task entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
         self.cn = 'usn_cleanup_' + Task._get_task_date()
         dn = "cn=" + self.cn + ",cn=USN tombstone cleanup task," + DN_TASKS
 
-        super(USNTombstoneCleanupTask, self).__init__(instance, dn, batch)
+        super(USNTombstoneCleanupTask, self).__init__(instance, dn)
 
     def _validate(self, rdn, properties, basedn):
         if not 'suffix' in properties and not 'backend' in properties:
@@ -90,12 +154,146 @@ class USNTombstoneCleanupTask(Task):
 
         return super(USNTombstoneCleanupTask, self)._validate(rdn, properties, basedn)
 
+
 class SchemaReloadTask(Task):
-    def __init__(self, instance, dn=None, batch=False):
+    """A single instance of schema reload task entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
         self.cn = 'schema_reload_' + Task._get_task_date()
         dn = "cn=" + self.cn + ",cn=schema reload task," + DN_TASKS
 
-        super(SchemaReloadTask, self).__init__(instance, dn, batch)
+        super(SchemaReloadTask, self).__init__(instance, dn)
+
+
+class AbortCleanAllRUVTask(Task):
+    """Abort the Clean All Ruv task on all masters. You should
+    call this from "CleanAllRUVTask.abort()" instead to provide
+    proper linking of the task information.
+
+    :param instance: The instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
+        self.cn = 'abortcleanallruv_' + Task._get_task_date()
+        dn = "cn=" + self.cn + ",cn=abort cleanallruv," + DN_TASKS
+
+        super(AbortCleanAllRUVTask, self).__init__(instance, dn)
+
+
+class CleanAllRUVTask(Task):
+    """Create the clean all ruv task. This will be replicated through
+    a topology to remove non-present ruvs. Note that if a ruv is NOT
+    able to be removed, this indicates a dangling replication agreement
+    on *some* master in the topology.
+
+    :param instance: The instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
+        self.cn = 'cleanallruv_' + Task._get_task_date()
+        dn = "cn=" + self.cn + ",cn=cleanallruv," + DN_TASKS
+        self._properties = None
+
+        super(CleanAllRUVTask, self).__init__(instance, dn)
+
+    def create(self, rdn=None, properties=None, basedn=None):
+        """Create the cleanallruvtask.
+
+        :param rdn: RDN of the new entry.
+        :type rdn: str
+        :param properties: Attributes for the new entry
+        :type properties: dict
+        :param basedn: Basedn to create the entry. Do not change this.
+        :type basedn: str
+        """
+
+        # Stash the props for abort
+        self._properties = properties
+        return super(CleanAllRUVTask, self).create(rdn, properties, basedn)
+
+    def abort(self, certify=False):
+        """Abort the current cleanallruvtask.
+
+        :param certify: certify abort on all masters
+        :type certify: bool
+        :returns: AbortCleanAllRUVTask
+        """
+
+        if certify is True:
+            self._properties['replica-certify-all'] = 'yes'
+        else:
+            self._properties['replica-certify-all'] = 'no'
+        # Then create the abort.
+        abort_task = AbortCleanAllRUVTask(self._instance)
+        abort_task.create(properties=self._properties)
+        return abort_task
+
+
+class ImportTask(Task):
+    """Create the import ldif task
+
+    :param instance: The instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
+        self.cn = 'import_' + Task._get_task_date()
+        dn = "cn=" + self.cn + ",cn=import," + DN_TASKS
+        self._properties = None
+
+        super(ImportTask, self).__init__(instance, dn)
+
+
+class ExportTask(Task):
+    """Create the export to ldif task
+
+    :param instance: The instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
+        self.cn = 'export_' + Task._get_task_date()
+        dn = "cn=" + self.cn + ",cn=export," + DN_TASKS
+        self._properties = None
+
+        super(ExportTask, self).__init__(instance, dn)
+
+
+class BackupTask(Task):
+    """Create the backup DB task
+
+    :param instance: The instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
+        self.cn = 'backup_' + Task._get_task_date()
+        dn = "cn=" + self.cn + ",cn=backup," + DN_TASKS
+        self._properties = None
+
+        super(BackupTask, self).__init__(instance, dn)
+
+
+class RestoreTask(Task):
+    """Create the restore DB task
+
+    :param instance: The instance
+    :type instance: lib389.DirSrv
+    """
+
+    def __init__(self, instance, dn=None):
+        self.cn = 'restore_' + Task._get_task_date()
+        dn = "cn=" + self.cn + ",cn=restore," + DN_TASKS
+        self._properties = None
+
+        super(RestoreTask, self).__init__(instance, dn)
+
 
 class Tasks(object):
     proxied_methods = 'search_s getEntry'.split()
@@ -124,7 +322,7 @@ class Tasks(object):
         dn = entry.dn
         while not done:
             entry = self.conn.getEntry(dn, attrlist=attrlist)
-            self.log.debug("task entry %r" % entry)
+            self.log.debug("task entry %r", entry)
 
             if entry.nsTaskExitCode:
                 exitCode = int(entry.nsTaskExitCode)
@@ -193,11 +391,11 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: import task %s for file %s exited with %d" %
-                           (cn, input_file, exitCode))
+            self.log.error("Error: import task %s for file %s exited with %d",
+                           cn, input_file, exitCode)
         else:
-            self.log.info("Import task %s for file %s completed successfully" %
-                          (cn, input_file))
+            self.log.info("Import task %s for file %s completed successfully",
+                          cn, input_file)
         self.dn = dn
         self.entry = entry
         return exitCode
@@ -262,11 +460,11 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: export task %s for file %s exited with %d" %
-                           (cn, output_file, exitCode))
+            self.log.error("Error: export task %s for file %s exited with %d",
+                           cn, output_file, exitCode)
         else:
-            self.log.info("Export task %s for file %s completed successfully" %
-                          (cn, output_file))
+            self.log.info("Export task %s for file %s completed successfully",
+                          cn, output_file)
 
         self.dn = dn
         self.entry = entry
@@ -306,7 +504,7 @@ class Tasks(object):
         try:
             self.conn.add_s(entry)
         except ldap.ALREADY_EXISTS:
-            self.log.error("Fail to add the backup task (%s)" % dn)
+            self.log.error("Fail to add the backup task (%s)", dn)
             return -1
 
         exitCode = 0
@@ -314,10 +512,10 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: backup task %s exited with %d" %
-                           (cn, exitCode))
+            self.log.error("Error: backup task %s exited with %d",
+                           cn, exitCode)
         else:
-            self.log.info("Backup task %s completed successfully" % (cn))
+            self.log.info("Backup task %s completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -368,7 +566,7 @@ class Tasks(object):
         try:
             self.conn.add_s(entry)
         except ldap.ALREADY_EXISTS:
-            self.log.error("Fail to add the backup task (%s)" % dn)
+            self.log.error("Fail to add the backup task (%s)", dn)
             return -1
 
         exitCode = 0
@@ -376,10 +574,10 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: restore task %s exited with %d" %
-                           (cn, exitCode))
+            self.log.error("Error: restore task %s exited with %d",
+                           cn, exitCode)
         else:
-            self.log.info("Restore task %s completed successfully" % (cn))
+            self.log.info("Restore task %s completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -467,7 +665,7 @@ class Tasks(object):
         try:
             self.conn.add_s(entry)
         except ldap.ALREADY_EXISTS:
-            self.log.error("Fail to add the index task for %s" % attrname)
+            self.log.error("Fail to add the index task for %s", attrname)
             return -1
 
         exitCode = 0
@@ -475,10 +673,10 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: index task %s exited with %d" %
-                           (cn, exitCode))
+            self.log.error("Error: index task %s exited with %d",
+                           cn, exitCode)
         else:
-            self.log.info("Index task %s completed successfully" % (cn))
+            self.log.info("Index task %s completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -554,12 +752,12 @@ class Tasks(object):
 
         if exitCode:
             self.log.error(
-                "Error: fixupMemberOf task %s for basedn %s exited with %d" %
-                (cn, suffix, exitCode))
+                "Error: fixupMemberOf task %s for basedn %s exited with %d",
+                cn, suffix, exitCode)
         else:
             self.log.info(
-                "fixupMemberOf task %s for basedn %s completed successfully" %
-                (cn, suffix))
+                "fixupMemberOf task %s for basedn %s completed successfully",
+                cn, suffix)
 
         self.dn = dn
         self.entry = entry
@@ -613,12 +811,14 @@ class Tasks(object):
 
         if exitCode:
             self.log.error(
-                "Error: tombstone fixup task %s for backend %s exited with %d"
-                % (cn, bename, exitCode))
+                "Error: tombstone fixup task %s for backend %s exited "
+                "with %d",
+                cn, bename, exitCode)
         else:
             self.log.info(
-                "tombstone fixup task %s for backend %s completed successfully"
-                % (cn, bename))
+                "tombstone fixup task %s for backend %s completed "
+                "successfully",
+                cn, bename)
 
         self.dn = dn
         self.entry = entry
@@ -661,12 +861,12 @@ class Tasks(object):
 
         if exitCode:
             self.log.error(
-                "Error: Automember Rebuild Membership task (%s) exited with %d"
-                % (cn, exitCode))
+                "Error: Automember Rebuild Membership task (%s) exited "
+                "with %d", cn, exitCode)
         else:
             self.log.info(
-                "Automember Rebuild Membership task(%s) completed successfully"
-                % (cn))
+                "Automember Rebuild Membership task(%s) completed"
+                "successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -714,12 +914,12 @@ class Tasks(object):
 
         if exitCode:
             self.log.error(
-                "Error: Automember Export Updates task (%s) exited with %d" %
-                (cn, exitCode))
+                "Error: Automember Export Updates task (%s) exited with %d",
+                cn, exitCode)
         else:
             self.log.info(
-                "Automember Export Updates task (%s) completed successfully" %
-                (cn))
+                "Automember Export Updates task (%s) completed successfully",
+                cn)
 
         self.dn = dn
         self.entry = entry
@@ -762,12 +962,11 @@ class Tasks(object):
 
         if exitCode:
             self.log.error(
-                "Error: Automember Map Updates task (%s) exited with %d" %
-                (cn, exitCode))
+                "Error: Automember Map Updates task (%s) exited with %d",
+                cn, exitCode)
         else:
             self.log.info(
-                "Automember Map Updates task (%s) completed successfully" %
-                (cn))
+                "Automember Map Updates task (%s) completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -805,12 +1004,11 @@ class Tasks(object):
 
         if exitCode:
             self.log.error(
-                "Error: Fixup Linked Attributes task (%s) exited with %d" %
-                (cn, exitCode))
+                "Error: Fixup Linked Attributes task (%s) exited with %d",
+                cn, exitCode)
         else:
             self.log.info(
-                "Fixup Linked Attributes task (%s) completed successfully" %
-                (cn))
+                "Fixup Linked Attributes task (%s) completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -846,11 +1044,10 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: Schema Reload task (%s) exited with %d" %
-                           (cn, exitCode))
+            self.log.error("Error: Schema Reload task (%s) exited with %d",
+                           cn, exitCode)
         else:
-            self.log.info("Schema Reload task (%s) completed successfully" %
-                          (cn))
+            self.log.info("Schema Reload task (%s) completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -890,11 +1087,11 @@ class Tasks(object):
 
         if exitCode:
             self.log.error(
-                "fixupWinsyncMembers 'memberuid task' (%s) exited with %d" %
-                (cn, exitCode))
+                "fixupWinsyncMembers 'memberuid task' (%s) exited with %d",
+                cn, exitCode)
         else:
             self.log.info(
-                "'memberuid task' (%s) completed successfully" % (cn))
+                "'memberuid task' (%s) completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -933,11 +1130,11 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: Syntax Validate (%s) exited with %d" %
-                           (cn, exitCode))
+            self.log.error("Error: Syntax Validate (%s) exited with %d",
+                           cn, exitCode)
         else:
-            self.log.info("Syntax Validate task (%s) completed successfully" %
-                          (cn))
+            self.log.info("Syntax Validate task (%s) completed successfully",
+                          cn)
 
         self.dn = dn
         self.entry = entry
@@ -982,12 +1179,11 @@ class Tasks(object):
 
         if exitCode:
             self.log.error(
-                "Error: USN tombstone cleanup task (%s) exited with %d" %
-                (cn, exitCode))
+                "Error: USN tombstone cleanup task (%s) exited with %d",
+                cn, exitCode)
         else:
             self.log.info(
-                "USN tombstone cleanup task (%s) completed successfully" %
-                (cn))
+                "USN tombstone cleanup task (%s) completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -1030,11 +1226,11 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: Sysconfig Reload task (%s) exited with %d" %
-                           (cn, exitCode))
+            self.log.error("Error: Sysconfig Reload task (%s) exited with %d",
+                           cn, exitCode)
         else:
-            self.log.info("Sysconfig Reload task (%s) completed successfully" %
-                          (cn))
+            self.log.info("Sysconfig Reload task (%s) completed successfully",
+                          cn)
 
         self.dn = dn
         self.entry = entry
@@ -1079,11 +1275,10 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: cleanAllRUV task (%s) exited with %d" %
-                           (cn, exitCode))
+            self.log.error("Error: cleanAllRUV task (%s) exited with %d",
+                           cn, exitCode)
         else:
-            self.log.info("cleanAllRUV task (%s) completed successfully" %
-                          (cn))
+            self.log.info("cleanAllRUV task (%s) completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -1133,12 +1328,11 @@ class Tasks(object):
 
         if exitCode:
             self.log.error(
-                "Error: Abort cleanAllRUV task (%s) exited with %d" %
-                (cn, exitCode))
+                "Error: Abort cleanAllRUV task (%s) exited with %d",
+                cn, exitCode)
         else:
             self.log.info(
-                "Abort cleanAllRUV task (%s) completed successfully" %
-                (cn))
+                "Abort cleanAllRUV task (%s) completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
@@ -1184,10 +1378,10 @@ class Tasks(object):
             (done, exitCode) = self.conn.tasks.checkTask(entry, True)
 
         if exitCode:
-            self.log.error("Error: upgradedb task (%s) exited with %d" %
-                           (cn, exitCode))
+            self.log.error("Error: upgradedb task (%s) exited with %d",
+                           cn, exitCode)
         else:
-            self.log.info("Upgradedb task (%s) completed successfully" % (cn))
+            self.log.info("Upgradedb task (%s) completed successfully", cn)
 
         self.dn = dn
         self.entry = entry
