@@ -32,12 +32,15 @@ import socket
 import time
 import sys
 import filecmp
+import six
+import shlex
 from socket import getfqdn
 from ldapurl import LDAPUrl
 from contextlib import closing
 
 import lib389
 from lib389.paths import Paths
+from lib389.dseldif import DSEldif
 from lib389._constants import (
         DEFAULT_USER, VALGRIND_WRAPPER, DN_CONFIG, CFGSUFFIX, LOCALHOST,
         ReplicaRole, CONSUMER_REPLICAID
@@ -50,19 +53,7 @@ from lib389.properties import (
 
 MAJOR, MINOR, _, _, _ = sys.version_info
 
-logging.basicConfig(level=logging.DEBUG)
 log = logging.getLogger(__name__)
-#
-# Decorator
-#
-
-
-def static_var(varname, value):
-    def decorate(func):
-        setattr(func, varname, value)
-        return func
-    return decorate
-
 
 #
 # Various searches to be used in getEntry
@@ -75,6 +66,105 @@ searches = {
                '(&(objectclass=glue)(objectclass=extensibleobject))', ['dn'])
 }
 
+# Map table for pseudolocalized strings
+_chars = {
+    " ": six.u("\u2003"),
+    "!": six.u("\u00a1"),
+    "\"": six.u("\u2033"),
+    "#": six.u("\u266f"),
+    "$": six.u("\u20ac"),
+    "%": six.u("\u2030"),
+    "&": six.u("\u214b"),
+    "'": six.u("\u00b4"),
+    ")": six.u("}"),
+    "(": six.u("{"),
+    "*": six.u("\u204e"),
+    "+": six.u("\u207a"),
+    ",": six.u("\u060c"),
+    "-": six.u("\u2010"),
+    ".": six.u("\u00b7"),
+    "/": six.u("\u2044"),
+    "0": six.u("\u24ea"),
+    "1": six.u("\u2460"),
+    "2": six.u("\u2461"),
+    "3": six.u("\u2462"),
+    "4": six.u("\u2463"),
+    "5": six.u("\u2464"),
+    "6": six.u("\u2465"),
+    "7": six.u("\u2466"),
+    "8": six.u("\u2467"),
+    "9": six.u("\u2468"),
+    ":": six.u("\u2236"),
+    ";": six.u("\u204f"),
+    "<": six.u("\u2264"),
+    "=": six.u("\u2242"),
+    ">": six.u("\u2265"),
+    "?": six.u("\u00bf"),
+    "@": six.u("\u055e"),
+    "A": six.u("\u00c5"),
+    "B": six.u("\u0181"),
+    "C": six.u("\u00c7"),
+    "D": six.u("\u00d0"),
+    "E": six.u("\u00c9"),
+    "F": six.u("\u0191"),
+    "G": six.u("\u011c"),
+    "H": six.u("\u0124"),
+    "I": six.u("\u00ce"),
+    "J": six.u("\u0134"),
+    "K": six.u("\u0136"),
+    "L": six.u("\u013b"),
+    "M": six.u("\u1e40"),
+    "N": six.u("\u00d1"),
+    "O": six.u("\u00d6"),
+    "P": six.u("\u00de"),
+    "Q": six.u("\u01ea"),
+    "R": six.u("\u0154"),
+    "S": six.u("\u0160"),
+    "T": six.u("\u0162"),
+    "U": six.u("\u00db"),
+    "V": six.u("\u1e7c"),
+    "W": six.u("\u0174"),
+    "X": six.u("\u1e8a"),
+    "Y": six.u("\u00dd"),
+    "Z": six.u("\u017d"),
+    "[": six.u("\u2045"),
+    "\\": six.u("\u2216"),
+    "]": six.u("\u2046"),
+    "^": six.u("\u02c4"),
+    "_": six.u("\u203f"),
+    "`": six.u("\u2035"),
+    "a": six.u("\u00e5"),
+    "b": six.u("\u0180"),
+    "c": six.u("\u00e7"),
+    "d": six.u("\u00f0"),
+    "e": six.u("\u00e9"),
+    "f": six.u("\u0192"),
+    "g": six.u("\u011d"),
+    "h": six.u("\u0125"),
+    "i": six.u("\u00ee"),
+    "j": six.u("\u0135"),
+    "k": six.u("\u0137"),
+    "l": six.u("\u013c"),
+    "m": six.u("\u0271"),
+    "n": six.u("\u00f1"),
+    "o": six.u("\u00f6"),
+    "p": six.u("\u00fe"),
+    "q": six.u("\u01eb"),
+    "r": six.u("\u0155"),
+    "s": six.u("\u0161"),
+    "t": six.u("\u0163"),
+    "u": six.u("\u00fb"),
+    "v": six.u("\u1e7d"),
+    "w": six.u("\u0175"),
+    "x": six.u("\u1e8b"),
+    "y": six.u("\u00fd"),
+    "z": six.u("\u017e"),
+    "{": six.u("("),
+    "}": six.u(")"),
+    "|": six.u("\u00a6"),
+    "~": six.u("\u02de"),
+}
+
 #
 # Utilities
 #
@@ -839,7 +929,12 @@ def ensure_bytes(val):
 
 def ensure_str(val):
     if val != None and type(val) != str:
-        return val.decode('utf-8')
+        try:
+            result = val.decode('utf-8')
+        except UnicodeDecodeError:
+            # binary value, just return str repr?
+            result = str(val)
+        return result
     return val
 
 
@@ -871,3 +966,62 @@ def ensure_dict_str(val):
         else:
             retdict[k] = ensure_str(val[k])
     return retdict
+
+
+def pseudolocalize(string):
+    pseudo_string = six.u("")
+    for char in string:
+        try:
+            pseudo_string += _chars[char]
+        except KeyError:
+            pseudo_string += char
+    return pseudo_string
+
+
+def assert_c(condition, msg="Assertion Failed"):
+    """This is the same as assert, but assert is compiled out
+    when optimisation is enabled. This prevents compiling out.
+    """
+    if not condition:
+        raise AssertionError(msg)
+
+
+def format_cmd_list(cmd):
+    """Format the subprocess command list to the quoted shell string"""
+
+    return " ".join(map(shlex.quote, cmd))
+
+
+def get_ldapurl_from_serverid(instance):
+    """ Take an instance name, and get the host/port/protocol from dse.ldif
+    and return a LDAP URL to use in the CLI tools (dsconf)
+
+    :param instance: The server ID of a server instance
+    :return tuple of LDAPURL and certificate directory (for LDAPS)
+    """
+    try:
+        dse_ldif = DSEldif(None, instance)
+    except:
+        return (None, None)
+
+    port = dse_ldif.get("cn=config", "nsslapd-port", single=True)
+    secureport = dse_ldif.get("cn=config", "nsslapd-secureport", single=True)
+    host = dse_ldif.get("cn=config", "nsslapd-localhost", single=True)
+    sec = dse_ldif.get("cn=config", "nsslapd-security", single=True)
+    ldapi_listen = dse_ldif.get("cn=config", "nsslapd-ldapilisten", single=True)
+    ldapi_autobind = dse_ldif.get("cn=config", "nsslapd-ldapiautobind", single=True)
+    ldapi_socket = dse_ldif.get("cn=config", "nsslapd-ldapifilepath", single=True)
+    certdir = dse_ldif.get("cn=config", "nsslapd-certdir", single=True)
+
+    if ldapi_listen is not None and ldapi_listen.lower() == "on" and \
+       ldapi_autobind is not None and ldapi_autobind.lower() == "on" and \
+       ldapi_socket is not None:
+        # Use LDAPI
+        socket = ldapi_socket.replace("/", "%2f")  # Escape the path
+        return ("ldapi://" + socket, None)
+    elif sec is not None and sec.lower() == "on" and secureport is not None:
+        # Use LDAPS
+        return ("ldaps://{}:{}".format(host, secureport), certdir)
+    else:
+        # Use LDAP
+        return ("ldap://{}:{}".format(host, port), None)
